CREATE TABLE  PLS_METER (
    PLS_METER_ID NUMBER NOT NULL,
    IP VARCHAR2(40) NOT NULL,
    PORT NUMBER NOT NULL,
    DEVICE_ADDRESS NUMBER NOT NULL,
    ID_PRIVEZ NUMBER NOT NULL,
    NAME VARCHAR2(100) NOT NULL,
    METER_NUMBER NUMBER NOT NULL,
    AMPERAGE NUMBER,
    PHASE_COUNT NUMBER,
    NPRIKLJ_SIFRA VARCHAR2(6) NOT NULL,
    PONTOON VARCHAR2(20),
    SOCKET VARCHAR2(20),
    ACTIVE_PLS_READING_ID NUMBER,
    LAST_PLS_METER_STATUS_ID NUMBER,
    ACTIVE VARCHAR2(1) NOT NULL
);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_PK PRIMARY KEY (PLS_METER_ID);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_NNPRIVEZ_FK FOREIGN KEY (ID_PRIVEZ) REFERENCES NNPRIVEZ(ID_PRIVEZ);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_NPRIKLJ_FK FOREIGN KEY (NPRIKLJ_SIFRA) REFERENCES NPRIKLJ(SIFRA);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_NUMBER_UQ UNIQUE(IP, PORT, DEVICE_ADDRESS, METER_NUMBER);

CREATE SEQUENCE PLS_METER_SEQ MINVALUE 1 INCREMENT BY 1 START WITH 1 NOCACHE NOORDER NOCYCLE;

CREATE TABLE  PLS_READING (
    PLS_READING_ID NUMBER NOT NULL,
    PLS_METER_ID NUMBER NOT NULL,
    ID_LASTNIKA NUMBER,
    ID_PLOVILA NUMBER,
    ID_PRIVEZ NUMBER,
    DT_READ_START DATE NOT NULL,
    DT_READ_END DATE,
    VALUE_START NUMBER NOT NULL,
    VALUE_END NUMBER NOT NULL
);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_PK PRIMARY KEY (PLS_READING_ID);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_METER_FK FOREIGN KEY (PLS_METER_ID) REFERENCES PLS_METER(PLS_METER_ID);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_NNPRIVEZ_FK FOREIGN KEY (ID_PRIVEZ) REFERENCES NNPRIVEZ(ID_PRIVEZ);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_DT_UQ UNIQUE(PLS_METER_ID, ID_PRIVEZ, DT_READ_START);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_KUPCI_FK FOREIGN KEY (ID_LASTNIKA) REFERENCES KUPCI(ID);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_PLOVILA_FK FOREIGN KEY (ID_PLOVILA) REFERENCES PLOVILA(ID);

ALTER TABLE PLS_READING ADD CONSTRAINT PLS_READING_CHK_OWNER CHECK (ID_LASTNIKA IS NOT NULL OR ID_PLOVILA IS NOT NULL OR ID_PRIVEZ IS NOT NULL);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_READING_FK FOREIGN KEY (ACTIVE_PLS_READING_ID) REFERENCES PLS_READING(PLS_READING_ID);

CREATE SEQUENCE PLS_READING_SEQ MINVALUE 1 INCREMENT BY 1 START WITH 1 NOCACHE NOORDER NOCYCLE;

CREATE TABLE PLS_METER_STATUS (
    PLS_METER_STATUS_ID NUMBER NOT NULL,
    PLS_METER_ID NUMBER NOT NULL,
    DT_READ DATE NOT NULL,
    VALUE NUMBER NOT NULL,
    ONLINE_STATUS VARCHAR2(1) NOT NULL,
    LOCK_STATUS VARCHAR2(1) NOT NULL,
    VERSION VARCHAR2(30),
    MESSAGE VARCHAR2(250)
);

ALTER TABLE PLS_METER_STATUS ADD CONSTRAINT PLS_METER_STATUS_PK PRIMARY KEY (PLS_METER_STATUS_ID);

ALTER TABLE PLS_METER_STATUS ADD CONSTRAINT PLS_METER_STATUS_METER_FK FOREIGN KEY (PLS_METER_ID) REFERENCES PLS_METER(PLS_METER_ID);

ALTER TABLE PLS_METER ADD CONSTRAINT PLS_METER_STATUS_FK FOREIGN KEY (LAST_PLS_METER_STATUS_ID) REFERENCES PLS_METER_STATUS(PLS_METER_STATUS_ID);

CREATE SEQUENCE PLS_METER_STATUS_SEQ MINVALUE 1 INCREMENT BY 1 START WITH 1 NOCACHE NOORDER  NOCYCLE;

CREATE OR REPLACE VIEW V_PLS_METER (PLS_METER_ID, IP, PORT, DEVICE_ADDRESS, ID_PRIVEZ, NAME, METER_NUMBER, AMPERAGE, PHASE_COUNT, NPRIKLJ_SIFRA, PONTOON, SOCKET, N_PRIVEZA,
        NPRIKLJ_INTERNI_OPIS, NPRIKLJ_OPIS, ACTIVE_PLS_READING_ID, LAST_PLS_METER_STATUS_ID, LAST_STATUS_UPDATE_DT, LAST_VALUE, LAST_ONLINE_STATUS, LAST_LOCK_STATUS, LAST_VERSION,
        ACTIVE) 
AS SELECT
    M.PLS_METER_ID,
    M.IP,
    M.PORT,
    M.DEVICE_ADDRESS,
    M.ID_PRIVEZ,
    M.NAME,
    M.METER_NUMBER,
    M.AMPERAGE,
    M.PHASE_COUNT,
    M.NPRIKLJ_SIFRA,
    M.PONTOON,
    M.SOCKET,
    P.N_PRIVEZA,
    VP.INTERNI_OPIS AS NPRIKLJ_INTERNI_OPIS,
    VP.OPIS AS NPRIKLJ_OPIS,
    M.ACTIVE_PLS_READING_ID,
    M.LAST_PLS_METER_STATUS_ID,
    MS.DT_READ AS LAST_STATUS_UPDATE_DT,
    MS.VALUE AS LAST_VALUE,
    MS.ONLINE_STATUS AS LAST_ONLINE_STATUS,
    MS.LOCK_STATUS AS LAST_LOCK_STATUS,
    MS.VERSION AS LAST_VERSION,
    M.ACTIVE
FROM 
    PLS_METER M
    LEFT OUTER JOIN NNPRIVEZ P ON P.ID_PRIVEZ = M.ID_PRIVEZ
    LEFT OUTER JOIN NPRIKLJ VP ON VP.SIFRA = M.NPRIKLJ_SIFRA
    LEFT OUTER JOIN PLS_METER_STATUS MS ON MS.PLS_METER_STATUS_ID = M.LAST_PLS_METER_STATUS_ID
;

CREATE OR REPLACE VIEW V_PLS_READING (ID, PLS_READING_ID, PLS_METER_ID, ID_PRIVEZ, DT_READ_START, VALUE_START, DT_READ_END, VALUE_END, METER_NAME, NPRIKLJ_SIFRA, AMPERAGE,
    PHASE_COUNT, IP, PORT, DEVICE_ADDRESS, METER_NUMBER, N_PRIVEZA, NPRIKLJ_INTERNI_OPIS, NPRIKLJ_OPIS, LAST_STATUS_UPDATE_DT, LAST_VALUE, LAST_ONLINE_STATUS, LAST_LOCK_STATUS,
    LAST_VERSION, OWNER_NAME, VESSEL_NAME, ACTIVE) AS
SELECT
    M.PLS_METER_ID AS ID,
    R.PLS_READING_ID,
    M.PLS_METER_ID,
    R.ID_PRIVEZ,
    R.DT_READ_START,
    R.VALUE_START,
    R.DT_READ_END,
    R.VALUE_END,
    M.NAME AS METER_NAME,
    M.NPRIKLJ_SIFRA,
    M.AMPERAGE,
    M.PHASE_COUNT,
    M.IP,
    M.PORT,
    M.DEVICE_ADDRESS,
    M.METER_NUMBER,
    P.N_PRIVEZA,
    VP.INTERNI_OPIS AS NPRIKLJ_INTERNI_OPIS,
    VP.OPIS AS NPRIKLJ_OPIS,
    MS.DT_READ AS LAST_STATUS_UPDATE_DT,
    MS.VALUE AS LAST_VALUE,
    MS.ONLINE_STATUS AS LAST_ONLINE_STATUS,
    MS.LOCK_STATUS AS LAST_LOCK_STATUS,
    MS.VERSION AS LAST_VERSION,
    (CASE WHEN R.ID_LASTNIKA IS NOT NULL THEN ((CASE WHEN (SELECT VREDNOST FROM S_NASTAVITVE WHERE FILTER = 'MARINA' AND SEKCIJA = 'MARINA' AND NAZIV = 'ChangeFirstLastName') = '1' THEN (K.IME || ' ' || K.PRIIMEK) ELSE (K.PRIIMEK || ' ' || K.IME) END)) ELSE NULL END) AS OWNER_NAME,
    PL.IME AS VESSEL_NAME,
    M.ACTIVE
FROM
    PLS_METER M
    LEFT OUTER JOIN PLS_READING R ON R.PLS_READING_ID = M.ACTIVE_PLS_READING_ID
    LEFT OUTER JOIN NNPRIVEZ P ON P.ID_PRIVEZ = R.ID_PRIVEZ
    LEFT OUTER JOIN NPRIKLJ VP ON VP.SIFRA = M.NPRIKLJ_SIFRA
    LEFT OUTER JOIN PLS_METER_STATUS MS ON MS.PLS_METER_STATUS_ID = M.LAST_PLS_METER_STATUS_ID
    LEFT OUTER JOIN KUPCI K ON K.ID = R.ID_LASTNIKA
    LEFT OUTER JOIN PLOVILA PL ON PL.ID = R.ID_PLOVILA
;

CREATE OR REPLACE VIEW V_PLS_READING_EXP (ID, PLS_READING_ID, PLS_METER_ID, ID_PRIVEZ, DT_READ_START, VALUE_START, DT_READ_END, VALUE_END, METER_NAME, NPRIKLJ_SIFRA, AMPERAGE,
    PHASE_COUNT, IP, PORT, DEVICE_ADDRESS, METER_NUMBER, N_PRIVEZA, NPRIKLJ_INTERNI_OPIS, NPRIKLJ_OPIS, LAST_STATUS_UPDATE_DT, LAST_VALUE, LAST_ONLINE_STATUS, LAST_LOCK_STATUS,
    LAST_VERSION, OWNER_NAME, VESSEL_NAME, ACTIVE) AS
SELECT
    R.PLS_READING_ID AS ID,
    R.PLS_READING_ID,
    M.PLS_METER_ID,
    R.ID_PRIVEZ,
    R.DT_READ_START,
    R.VALUE_START,
    R.DT_READ_END,
    R.VALUE_END,
    M.NAME AS METER_NAME,
    M.NPRIKLJ_SIFRA,
    M.AMPERAGE,
    M.PHASE_COUNT,
    M.IP,
    M.PORT,
    M.DEVICE_ADDRESS,
    M.METER_NUMBER,
    P.N_PRIVEZA,
    VP.INTERNI_OPIS AS NPRIKLJ_INTERNI_OPIS,
    VP.OPIS AS NPRIKLJ_OPIS,
    MS.DT_READ AS LAST_STATUS_UPDATE_DT,
    MS.VALUE AS LAST_VALUE,
    MS.ONLINE_STATUS AS LAST_ONLINE_STATUS,
    MS.LOCK_STATUS AS LAST_LOCK_STATUS,
    MS.VERSION AS LAST_VERSION,
    (CASE WHEN R.ID_LASTNIKA IS NOT NULL THEN ((CASE WHEN (SELECT VREDNOST FROM S_NASTAVITVE WHERE FILTER = 'MARINA' AND SEKCIJA = 'MARINA' AND NAZIV = 'ChangeFirstLastName') = '1' THEN (K.IME || ' ' || K.PRIIMEK) ELSE (K.PRIIMEK || ' ' || K.IME) END)) ELSE NULL END) AS OWNER_NAME,
    PL.IME AS VESSEL_NAME,
    M.ACTIVE
FROM
    PLS_READING R
    JOIN PLS_METER M ON M.PLS_METER_ID = R.PLS_METER_ID
    LEFT OUTER JOIN NNPRIVEZ P ON P.ID_PRIVEZ = R.ID_PRIVEZ
    LEFT OUTER JOIN NPRIKLJ VP ON VP.SIFRA = M.NPRIKLJ_SIFRA
    LEFT OUTER JOIN PLS_METER_STATUS MS ON MS.PLS_METER_STATUS_ID = M.LAST_PLS_METER_STATUS_ID
    LEFT OUTER JOIN KUPCI K ON K.ID = R.ID_LASTNIKA
    LEFT OUTER JOIN PLOVILA PL ON PL.ID = R.ID_PLOVILA
;

insert into verzije_baza
(id, dt_vpis, verzija, ver_nr, opis) values (1856, SYSDATE, '5.5', 1776, 'PULS')
;
